@@ -5,6 +5,7 @@ from inspect import (
     isawaitable,
     iscoroutinefunction,
 )
+import inspect
 from typing import Iterable, Callable, Union, Mapping
 from functools import partial, wraps, update_wrapper
 from json import JSONDecodeError, JSONEncoder, dumps
@@ -15,11 +16,15 @@ from typing import Awaitable, get_origin
 from collections.abc import Awaitable as _Awaitable
 import os
 
-from i2.signatures import set_signature_of_func, ch_signature_to_all_pk, Sig
-from i2.deco import ch_func_to_all_pk
+from i2.signatures import (
+    set_signature_of_func,
+    ch_signature_to_all_pk,
+    Sig,
+    ch_func_to_all_pk,
+)
 from i2.errors import ModuleNotFoundIgnore
 
-from py2http.schema_tools import validate_input
+from py2http.schema_tools import mk_input_schema_from_func, validate_input
 from py2http.types import (
     WriteOpResult,
     ParameterKind,
@@ -569,7 +574,7 @@ class Decora(Decorator):
             cls.__new__ = __new__
 
 
-from i2.deco import copy_func, params_of
+from i2.signatures import copy_func, params_of
 
 
 def replace_with_params(target=None, /, *, source=None, inplace=False):
@@ -968,43 +973,53 @@ def route(route_name):
     return add_attrs(route=route_name)
 
 
-def validate_and_invoke_mapper(func, inputs, request_schema):
+def validate_and_invoke_mapper(func, inputs):
+    request_schema = getattr(func, 'request_schema', None)
     if request_schema:
         validate_input(inputs, request_schema)
-    return func(inputs)
+    return func(**inputs)
 
 
 def handle_json_req(func):
-    def input_mapper(req, request_schema):
+    func.request_schema = mk_input_schema_from_func(func)
+    func.content_type = 'json'
+
+    @wraps(func)
+    def input_mapper(req):
         inputs = _get_req_inputs(
             req, getattr(req, 'get_json', getattr(req, 'json', None))
         )
-        return validate_and_invoke_mapper(func, inputs, request_schema)
+        return validate_and_invoke_mapper(func, inputs)
 
-    input_mapper.content_type = 'json'
     return input_mapper
 
 
 def handle_multipart_req(func):
+    func.request_schema = mk_input_schema_from_func(func)
+    func.content_type = 'multipart'
+
     # TODO: make this work with Bottle
-    def input_mapper(req, request_schema):
+    @wraps(func)
+    def input_mapper(req):
         inputs = _get_req_inputs(req, req.post)
-        return validate_and_invoke_mapper(func, inputs, request_schema)
+        return validate_and_invoke_mapper(func, inputs)
 
-    input_mapper.content_type = 'multipart'
     return input_mapper
 
 
 def handle_raw_req(func):
-    def input_mapper(req, request_schema):
+    func.request_schema = mk_input_schema_from_func(func)
+    func.content_type = 'raw'
+
+    @wraps(func)
+    def input_mapper(req):
         inputs = _get_req_inputs(req, req.text)
-        return validate_and_invoke_mapper(func, inputs, request_schema)
+        return validate_and_invoke_mapper(func, inputs)
 
-    input_mapper.content_type = 'raw'
     return input_mapper
 
 
-def base_output_mapper(output, input_kwargs):
+def base_output_mapper(output, **inputs):
     return output
 
 
@@ -1060,16 +1075,16 @@ def send_json_resp(func):
     framework = os.getenv('PY2HTTP_FRAMEWORK', BOTTLE)
     if framework == AIOHTTP:
 
-        async def output_mapper(output, input_kwargs):
-            mapped_output = func(output, input_kwargs)
+        async def output_mapper(output, **input_kwargs):
+            mapped_output = func(output, **input_kwargs)
             if isawaitable(mapped_output):
                 mapped_output = await mapped_output
             return web.json_response(mapped_output, dumps=JsonRespEncoder().encode)
 
     else:
 
-        def output_mapper(output, input_kwargs):
-            mapped_output = func(output, input_kwargs)
+        def output_mapper(output, **input_kwargs):
+            mapped_output = func(output, **input_kwargs)
             return dumps(mapped_output, cls=JsonRespEncoder)
 
     output_mapper.content_type = 'application/json'
@@ -1078,7 +1093,7 @@ def send_json_resp(func):
 
 def send_html_resp(func):
     # TODO bottle support
-    # async def output_mapper(output, input_kwargs):
+    # async def output_mapper(output, **input_kwargs):
     #     mapped_output = func(output, input_kwargs)
     #     if isawaitable(mapped_output):
     #         mapped_output = await mapped_output
@@ -1092,7 +1107,7 @@ def send_binary_resp(func):
     # TODO async support
     BINARY_CONTENT_TYPE = 'application/octet-stream'
 
-    def output_mapper(output, input_kwargs):
+    def output_mapper(output, **input_kwargs):
         from bottle import response
 
         mapped_output = func(output, input_kwargs)
@@ -1130,3 +1145,36 @@ def _get_req_inputs(req, get_body_func):
     if getattr(req, 'token', None):
         kwargs = dict(kwargs, **req.token)
     return kwargs
+
+
+def mk_handlers(methods: Iterable, decorator=None):
+    def get_class_that_defined_method(meth):
+        if inspect.ismethod(meth):
+            for cls in inspect.getmro(meth.__self__.__class__):
+                if meth.__name__ in cls.__dict__:
+                    return cls
+            meth = meth.__func__  # fallback to __qualname__ parsing
+        if inspect.isfunction(meth):
+            cls = getattr(
+                inspect.getmodule(meth),
+                meth.__qualname__.split('.<locals>', 1)[0].rsplit('.', 1)[0],
+                None,
+            )
+            if isinstance(cls, type):
+                sub_classes = cls.__subclasses__()
+                if len(sub_classes) == 1:
+                    return sub_classes[0]
+                return cls
+        return None
+
+    if not decorator:
+        decorator = lambda x: x
+    handlers = []
+    for item in methods:
+        has_mappers = isinstance(item, dict)
+        method = item['endpoint'] if has_mappers else item
+        cls = get_class_that_defined_method(method)
+        endpoint = decorator(mk_flat(cls, method, func_name=method.__name__))
+        handler = dict(item, endpoint=endpoint) if has_mappers else endpoint
+        handlers.append(handler)
+    return handlers
