@@ -92,7 +92,7 @@ def default_error_handler(error: Exception):
 
 
 default_configs = {
-    'app_name': 'OtoSense',
+    'app_name': 'HTTP Service',
     'framework': BOTTLE,
     'input_mapper': default_input_mapper,
     'output_mapper': default_output_mapper,
@@ -110,4 +110,7 @@ default_configs = {
     'cors_allowed_origins': '*',
     'publish_openapi': False,
     'openapi_insecure': False,
+    'publish_swagger': False,
+    'swagger_url': '/swagger',
+    'swagger_title': 'Swagger',
 }
